@@ -4,7 +4,7 @@ from pykechain.models.part import Part
 from pykechain.models.property import Property
 
 
-class ReferenceProperty(Property):
+class ReferenceProperty(Property):  # pragma: no cover
     """A virtual object representing a KE-chain reference property."""
 
     @property
