@@ -1,18 +1,20 @@
-import datetime
-import warnings
-from typing import Any, Union, Text, Iterable, Dict, Optional, List  # noqa: F401
+from datetime import datetime
+from typing import Union, Text, Dict, Optional, List  # noqa: F401
 
 import requests
-from six import text_type, string_types
 
 from pykechain.defaults import API_EXTRA_PARAMS
-from pykechain.enums import Multiplicity, ScopeStatus, SubprocessDisplayMode, KEChainPages
+from pykechain.enums import Multiplicity, ScopeStatus, SubprocessDisplayMode, KEChainPages, ScopeRoles, \
+    ScopeMemberActions
 from pykechain.exceptions import APIError, NotFoundError, IllegalArgumentError
 from pykechain.models.base import Base
+from pykechain.models.input_checks import check_text, check_datetime, check_enum, check_list_of_text, \
+    check_base, check_type
+from pykechain.models.representations.component import RepresentationsComponent
 from pykechain.models.sidebar.sidebar_manager import SideBarManager
 from pykechain.models.tags import TagsMixin
 from pykechain.models.team import Team
-from pykechain.utils import parse_datetime, is_uuid
+from pykechain.utils import parse_datetime, find
 
 
 class Scope2(Base, TagsMixin):
@@ -36,10 +38,9 @@ class Scope2(Base, TagsMixin):
     :type type: basestring
     """
 
-    def __init__(self, json, **kwargs):
-        # type: (dict, **Any) -> None
+    def __init__(self, json: Dict, **kwargs) -> None:
         """Construct a scope from provided json data."""
-        super(Scope2, self).__init__(json, **kwargs)
+        super().__init__(json, **kwargs)
 
         # for 'kechain2.core.wim <2.0.0'
         self.process = json.get('process')
@@ -56,14 +57,11 @@ class Scope2(Base, TagsMixin):
         self.start_date = parse_datetime(json.get('start_date'))
         self.due_date = parse_datetime(json.get('due_date'))
 
-    @property
-    def bucket(self):
-        """Bucket of the scope is deprecated in version 2.
-
-        .. deprecated:: 3.0
-           A `bucket` is a deprecated concept in KE-chain 3 backends. Use `scope_id` instead.
-        """
-        raise DeprecationWarning("Bucket has been deprecated in scope version 2")
+        self._representations_container = RepresentationsComponent(
+            self,
+            self.options.get('representations', {}),
+            self._save_representations,
+        )
 
     @property
     def team(self):
@@ -88,65 +86,70 @@ class Scope2(Base, TagsMixin):
 
     def refresh(self, json=None, url=None, extra_params=None):
         """Refresh the object in place."""
-        super(Scope2, self).refresh(json=json,
-                                    url=self._client._build_url('scope2', scope_id=self.id),
-                                    extra_params=API_EXTRA_PARAMS['scope2'])
+        super().refresh(json=json,
+                        url=self._client._build_url('scope2', scope_id=self.id),
+                        extra_params=API_EXTRA_PARAMS['scope2'])
 
+    @property
+    def representations(self):
+        """Get and set the scope representations."""
+        return self._representations_container.get_representations()
+
+    @representations.setter
+    def representations(self, value):
+        self._representations_container.set_representations(value)
+
+    def _save_representations(self, representation_options):
+        options = self.options
+        options.update({'representations': representation_options})
+        self.options = options
     #
     # CRUD methods
     #
 
-    def _update_scope_project_team(self, select_action, user, user_type):
+    def _update_scope_project_team(self, action, role, user):
         """
         Update the Project Team of the Scope. Updates include addition or removing of managers or members.
 
-        :param select_action: type of action to be applied
-        :type select_action: basestring
+        :param action: type of action to be applied
+        :type action: ScopeMemberActions
+        :param role: type of role to be applied to the user
+        :type role: ScopeRoles
         :param user: the username of the user to which the action applies to
         :type user: basestring
-        :param user_type: the type of the user (member or manager)
-        :type user_type: basestring
         :raises APIError: When unable to update the scope project team.
         """
-        if isinstance(user, (string_types, text_type)):
-            users = self._client._retrieve_users()
-            user_object = next((item for item in users['results'] if item["username"] == user), None)
-            if user_object:
-                url = self._client._build_url('scope2_{}'.format(select_action), scope_id=self.id)
+        action = check_enum(action, ScopeMemberActions, 'action')
+        role = check_enum(role, ScopeRoles, 'role')
+        user = check_text(user, 'user')
 
-                response = self._client._request('PUT', url,
-                                                 params=API_EXTRA_PARAMS[self.__class__.__name__.lower()],
-                                                 data={'user_id': user_object['pk']})
-                if response.status_code != requests.codes.ok:  # pragma: no cover
-                    raise APIError("Could not {} {} in Scope".format(select_action.split('_')[0], user_type))
+        users = self._client._retrieve_users()['results']  # type: List[Dict]
+        user_object = find(users, lambda u: u['username'] == user)  # type: Dict
+        if user_object is None:
+            raise NotFoundError('User "{}" does not exist'.format(user))
 
-                self._json_data = response.json().get('results')[0]
-            else:
-                raise NotFoundError("User {} does not exist".format(user))
-        else:
-            raise TypeError("User {} should be defined as a string".format(user))
+        url = self._client._build_url('scope2_{}_{}'.format(action, role), scope_id=self.id)
 
-    def _edit(self, update_dict):
-        if update_dict.get('options'):
-            update_dict['scope_options'] = update_dict.get('options')
-            del update_dict['options']
-        if update_dict.get('team'):
-            update_dict['team_id'] = update_dict.get('team')
-            del update_dict['team']
-
-        url = self._client._build_url('scope2', scope_id=self.id)
-
-        response = self._client._request('PUT', url, params=API_EXTRA_PARAMS[self.__class__.__name__.lower()],
-                                         json=update_dict)
+        response = self._client._request('PUT', url,
+                                         params=API_EXTRA_PARAMS[self.__class__.__name__.lower()],
+                                         data={'user_id': user_object['pk']})
 
         if response.status_code != requests.codes.ok:  # pragma: no cover
-            raise APIError("Could not update Scope ({})".format(response))
+            raise APIError("Could not {} {} in Scope".format(action, role), response=response)
 
         self.refresh(json=response.json().get('results')[0])
 
-    def edit(self, name=None, description=None, start_date=None, due_date=None, status=None, tags=None, team=None,
-             options=None):
-        # type: (Text, Text, datetime.datetime, datetime.datetime, Union[ScopeStatus, Text], Iterable[Text], Team, Dict) -> None  # noqa: E501
+    def edit(
+            self,
+            name: Optional[Text] = None,
+            description: Optional[Text] = None,
+            start_date: Optional[datetime] = None,
+            due_date: Optional[datetime] = None,
+            status: Optional[Union[Text, ScopeStatus]] = None,
+            tags: Optional[List[Text]] = None,
+            team: Optional[Union[Team, Text]] = None,
+            options: Optional[Dict] = None,
+    ) -> None:
         """Edit the details of a scope.
 
         :param name: (optionally) edit the name of the scope
@@ -176,7 +179,7 @@ class Scope2(Base, TagsMixin):
         >>> from datetime import datetime
         >>> project.edit(name='New project name',
         ...              description='Changing the description just because I can',
-        ...              start_date=datetime.utcnow(),  # naive time is interpreted as UTC time
+        ...              start_date=datetime.now(),  # naive time is interpreted as UTC time
         ...              status=ScopeStatus.CLOSED)
 
         If we want to provide timezone aware datetime objects we can use the 3rd party convenience library :mod:`pytz`.
@@ -201,73 +204,43 @@ class Scope2(Base, TagsMixin):
         >>> project.edit(team=my_team)
 
         """
-        update_dict = {'id': self.id}
-        if name is not None:
-            if isinstance(name, (str, text_type)):
-                update_dict.update({'name': name})
-                self.name = name
-            else:
-                raise IllegalArgumentError('Name should be a string')
-
-        if description is not None:  # isinstance(description, (str, text_type)):
-            if isinstance(description, (str, text_type)):
-                update_dict.update({'text': description})
-                self.text = description
-            else:
-                raise IllegalArgumentError('Description should be a string')
-
-        if start_date is not None:
-            if isinstance(start_date, datetime.datetime):
-                if not start_date.tzinfo:
-                    warnings.warn("The startdate '{}' is naive and not timezone aware, use pytz.timezone info. "
-                                  "This date is interpreted as UTC time.".format(start_date.isoformat(sep=' ')))
-                update_dict.update({'start_date': start_date.isoformat(sep='T')})
-            else:
-                raise IllegalArgumentError('Start date should be a datetime.datetime() object')
-
-        if due_date is not None:
-            if isinstance(due_date, datetime.datetime):
-                if not due_date.tzinfo:
-                    warnings.warn("The duedate '{}' is naive and not timezone aware, use pytz.timezone info. "
-                                  "This date is interpreted as UTC time.".format(due_date.isoformat(sep=' ')))
-                update_dict.update({'due_date': due_date.isoformat(sep='T')})
-            else:
-                raise IllegalArgumentError('Due date should be a datetime.datetime() object')
-
-        if status is not None:
-            if isinstance(status, (str, text_type)) and status in ScopeStatus.values():
-                update_dict.update({'status': status})
-            else:
-                raise IllegalArgumentError('Status should be a string and in the list of acceptable '
-                                           'status strings: {}'.format(ScopeStatus.values()))
-
+        update_dict = {
+            'id': self.id,
+            'name': check_text(name, 'name') or self.name,
+            'text': check_text(description, 'description') or self.description,
+            'start_date': check_datetime(start_date, 'start_date'),
+            'due_date': check_datetime(due_date, 'due_date'),
+            'status': check_enum(status, ScopeStatus, 'status') or self.status,
+        }
+        team = check_base(team, Team, 'team', method=self._client.team)
+        if team:
+            update_dict['team_id'] = team
+        tags = check_list_of_text(tags, 'tags', True)
         if tags is not None:
-            if isinstance(tags, (list, tuple, set)):
-                update_dict.update({'tags': tags})
-            else:
-                raise IllegalArgumentError('tags should be a an array (list, tuple, set) of strings')
+            update_dict['tags'] = tags
+        scope_options = check_type(options, dict, 'options')
+        if scope_options:
+            update_dict['scope_options'] = scope_options
 
-        if team is not None:
-            if isinstance(team, (str, text_type)) and is_uuid(team):
-                update_dict.update({'team_id': team})
-            elif isinstance(team, Team):
-                update_dict.update({'team_id': team.id})
-            else:
-                raise IllegalArgumentError("team should be the uuid of a team")
+        url = self._client._build_url('scope2', scope_id=self.id)
 
-        if options is not None:
-            if isinstance(options, dict):
-                update_dict.update({'options': options})
-            else:
-                raise IllegalArgumentError("options should be a dictionary")
+        response = self._client._request('PUT', url,
+                                         params=API_EXTRA_PARAMS[self.__class__.__name__.lower()],
+                                         json=update_dict)
 
-        # do the update itself in an abstracted function.
-        self._edit(update_dict)
+        if response.status_code != requests.codes.ok:  # pragma: no cover
+            raise APIError("Could not update Scope {}".format(self), response=response)
 
-    def clone(self, *args, **kwargs):
+        self.refresh(json=response.json().get('results')[0])
+
+        # TODO tags that are set are not in response
+        if tags is not None:
+            self._tags = tags
+
+    def clone(self, **kwargs):
         """Clone a scope.
 
-        See :method:`pykechain.Client.clone_scope()` for available paramters.
+        See :method:`pykechain.Client.clone_scope()` for available parameters.
         """
         return self._client.clone_scope(source_scope=self, **kwargs)
 
@@ -353,30 +326,21 @@ class Scope2(Base, TagsMixin):
 
         See :class:`pykechain.Client.activities` for available parameters.
         """
-        if self._client.match_app_version(label='wim', version='<2.0.0', default=True):
-            return self._client.activities(*args, scope=self.id, **kwargs)
-        else:
-            return self._client.activities(*args, scope_id=self.id, **kwargs)
+        return self._client.activities(*args, scope_id=self.id, **kwargs)
 
     def activity(self, *args, **kwargs):
         """Retrieve a single activity belonging to this scope.
 
         See :class:`pykechain.Client.activity` for available parameters.
         """
-        if self._client.match_app_version(label='wim', version='<2.0.0', default=True):
-            return self._client.activity(*args, scope=self.id, **kwargs)
-        else:
-            return self._client.activity(*args, scope_id=self.id, **kwargs)
+        return self._client.activity(*args, scope_id=self.id, **kwargs)
 
     def create_activity(self, *args, **kwargs):
         """Create a new activity belonging to this scope.
 
         See :class:`pykechain.Client.create_activity` for available parameters.
         """
-        if self._client.match_app_version(label='wim', version='<2.0.0', default=True):
-            return self._client.create_activity(self.process, *args, **kwargs)
-        else:
-            return self._client.create_activity(self.workflow_root, *args, **kwargs)
+        return self._client.create_activity(self.workflow_root, *args, **kwargs)
 
     def side_bar(self, *args, **kwargs) -> Optional[SideBarManager]:
         """Retrieve the side-bar manager."""
@@ -384,7 +348,8 @@ class Scope2(Base, TagsMixin):
 
     def set_landing_page(self,
                          activity: Union['Activity2', KEChainPages],
-                         task_display_mode: Optional[SubprocessDisplayMode] = SubprocessDisplayMode.ACTIVITIES) -> None:
+                         task_display_mode: Optional[SubprocessDisplayMode] = SubprocessDisplayMode.ACTIVITIES,
+                         ) -> None:
         """
         Update the landing page of the scope.
 
@@ -401,9 +366,7 @@ class Scope2(Base, TagsMixin):
             raise IllegalArgumentError(
                 'activity must be of class Activity2 or a KEChainPages option, "{}" is not.'.format(activity))
 
-        if task_display_mode not in SubprocessDisplayMode.values():
-            raise IllegalArgumentError('task_display_mode must be a WorkBreakdownDisplayMode option, '
-                                       '"{}" is not.'.format(task_display_mode))
+        check_enum(task_display_mode, SubprocessDisplayMode, 'task_display_mode')
 
         if isinstance(activity, Activity2):
             url = '#/scopes/{}/{}/{}'.format(self.id, task_display_mode, activity.id)
@@ -467,8 +430,7 @@ class Scope2(Base, TagsMixin):
     # User and Members of the Scope
     #
 
-    def members(self, is_manager=None, is_leadmember=None):
-        # type: (Optional[bool], Optional[bool]) -> List[Dict]
+    def members(self, is_manager: Optional[bool] = None, is_leadmember: Optional[bool] = None) -> List[Dict]:
         """
         Retrieve members of the scope.
 
@@ -494,8 +456,7 @@ class Scope2(Base, TagsMixin):
 
         return members
 
-    def add_member(self, member):
-        # type: (Text) -> None
+    def add_member(self, member: Text) -> None:
         """
         Add a single member to the scope.
 
@@ -505,12 +466,9 @@ class Scope2(Base, TagsMixin):
         :type member: basestring
         :raises APIError: when unable to update the scope member
         """
-        select_action = 'add_member'
+        self._update_scope_project_team(action=ScopeMemberActions.ADD, role=ScopeRoles.MEMBER, user=member)
 
-        self._update_scope_project_team(select_action=select_action, user=member, user_type='member')
-
-    def remove_member(self, member):
-        # type: (Text) -> None
+    def remove_member(self, member: Text) -> None:
         """
         Remove a single member to the scope.
 
@@ -518,12 +476,9 @@ class Scope2(Base, TagsMixin):
         :type member: basestring
         :raises APIError: when unable to update the scope member
         """
-        select_action = 'remove_member'
+        self._update_scope_project_team(action=ScopeMemberActions.REMOVE, role=ScopeRoles.MEMBER, user=member)
 
-        self._update_scope_project_team(select_action=select_action, user=member, user_type='member')
-
-    def add_manager(self, manager):
-        # type: (Text) -> None
+    def add_manager(self, manager: Text) -> None:
         """
         Add a single manager to the scope.
 
@@ -531,12 +486,9 @@ class Scope2(Base, TagsMixin):
         :type manager: basestring
         :raises APIError: when unable to update the scope manager
         """
-        select_action = 'add_manager'
+        self._update_scope_project_team(action=ScopeMemberActions.ADD, role=ScopeRoles.MANAGER, user=manager)
 
-        self._update_scope_project_team(select_action=select_action, user=manager, user_type='manager')
-
-    def remove_manager(self, manager):
-        # type: (Text) -> None
+    def remove_manager(self, manager: Text) -> None:
         """
         Remove a single manager to the scope.
 
@@ -544,12 +496,9 @@ class Scope2(Base, TagsMixin):
         :type manager: basestring
         :raises APIError: when unable to update the scope manager
         """
-        select_action = 'remove_manager'
+        self._update_scope_project_team(action=ScopeMemberActions.REMOVE, role=ScopeRoles.MANAGER, user=manager)
 
-        self._update_scope_project_team(select_action=select_action, user=manager, user_type='manager')
-
-    def add_leadmember(self, leadmember):
-        # type: (Text) -> None
+    def add_leadmember(self, leadmember: Text) -> None:
         """
         Add a single leadmember to the scope.
 
@@ -557,12 +506,9 @@ class Scope2(Base, TagsMixin):
         :type leadmember: basestring
         :raises APIError: when unable to update the scope leadmember
         """
-        select_action = 'add_leadmember'
+        self._update_scope_project_team(action=ScopeMemberActions.ADD, role=ScopeRoles.LEADMEMBER, user=leadmember)
 
-        self._update_scope_project_team(select_action=select_action, user=leadmember, user_type='leadmember')
-
-    def remove_leadmember(self, leadmember):
-        # type: (Text) -> None
+    def remove_leadmember(self, leadmember: Text) -> None:
         """
         Remove a single leadmember to the scope.
 
@@ -570,6 +516,4 @@ class Scope2(Base, TagsMixin):
         :type leadmember: basestring
         :raises APIError: when unable to update the scope leadmember
         """
-        select_action = 'remove_leadmember'
-
-        self._update_scope_project_team(select_action=select_action, user=leadmember, user_type='leadmember')
+        self._update_scope_project_team(action=ScopeMemberActions.REMOVE, role=ScopeRoles.LEADMEMBER, user=leadmember)
