@@ -2,17 +2,28 @@ from unittest import skipIf
 
 from pykechain.enums import Multiplicity, Category, Classification
 from pykechain.exceptions import NotFoundError, MultipleFoundError, APIError, IllegalArgumentError
-from pykechain.models import Part, PartSet, Part2
+from pykechain.models import PartSet, Part2
 from tests.classes import TestBetamax
 from tests.utils import TEST_FLAG_IS_PIM3
 
 
 class TestParts(TestBetamax):
+
+    def setUp(self):
+        super().setUp()
+        self._part = None
+
+    def tearDown(self):
+        if self._part:
+            self._part.delete()
+        super().tearDown()
+
     def test_retrieve_parts(self):
         parts = self.project.parts()
 
         # Check if there are parts
         self.assertTrue(len(parts))
+        self.assertIsInstance(parts[0], Part2)
 
     def test_retrieve_single_part(self):
         part_to_retrieve = self.project.parts()[0]
@@ -90,7 +101,7 @@ class TestParts(TestBetamax):
         bike_model = self.project.model(name='Bike')
 
         # testing
-        with self.assertRaises(APIError):
+        with self.assertRaises(IllegalArgumentError):
             self.client.create_model(name='Multiplicity does not exist', parent=bike_model, multiplicity='TEN')
 
     def test_part_add_delete_part(self):
@@ -112,7 +123,8 @@ class TestParts(TestBetamax):
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            self.client.create_part(name='Parent should be instance', parent=bike_model, model=bike_model)
+            self._part = self.client.create_part(name='Parent should be instance',
+                                                 parent=bike_model, model=bike_model)
 
     def test_create_part_where_model_is_instance(self):
         # setUp
@@ -120,7 +132,8 @@ class TestParts(TestBetamax):
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            self.client.create_part(name='Model should not be instance', parent=bike_instance, model=bike_instance)
+            self._part = self.client.create_part(name='Model should not be instance',
+                                                 parent=bike_instance, model=bike_instance)
 
     def test_create_model_where_parent_is_instance(self):
         # setUp
@@ -128,7 +141,8 @@ class TestParts(TestBetamax):
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            self.client.create_model(name='Parent should be model', parent=bike_instance, multiplicity=Multiplicity.ONE)
+            self._part = self.client.create_model(name='Parent should be model',
+                                                  parent=bike_instance, multiplicity=Multiplicity.ONE)
 
     def test_create_proxy_model_where_model_is_instance(self):
         # setUp
@@ -136,8 +150,8 @@ class TestParts(TestBetamax):
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            self.client.create_proxy_model(name='Model should not be instance', model=bike_instance,
-                                           parent=bike_instance)
+            self._part = self.client.create_proxy_model(name='Model should not be instance',
+                                                        model=bike_instance, parent=bike_instance)
 
     def test_create_proxy_model_where_parent_is_instance(self):
         # setUp
@@ -146,7 +160,8 @@ class TestParts(TestBetamax):
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            self.client.create_proxy_model(name='Parent should not be instance', model=bike_model, parent=bike_instance)
+            self._part = self.client.create_proxy_model(name='Parent should not be instance',
+                                                        model=bike_model, parent=bike_instance)
 
     def test_add_to_wrong_categories(self):
         # This test has the purpose of testing of whether APIErrors are raised when illegal operations are
@@ -159,21 +174,21 @@ class TestParts(TestBetamax):
         wheel_model = project.model('Wheel')
 
         with self.assertRaises(APIError):
-            bike_model.add(wheel_model)
+            self._part = bike_model.add(wheel_model)
 
         front_fork_instance = project.part('Front Fork')
 
         with self.assertRaises(APIError):
-            front_fork_instance.add_to(bike_instance)
+            self._part = front_fork_instance.add_to(bike_instance)
 
         with self.assertRaises(APIError):
-            bike_instance.add_model('Engine')
+            self._part = bike_instance.add_model('Engine')
 
         with self.assertRaises(APIError):
-            bike_instance.add_property('Electrical Power')
+            self._part = bike_instance.add_property('Electrical Power')
 
         with self.assertRaises(APIError):
-            bike_model.add_with_properties(wheel_model)
+            self._part = bike_model.add_with_properties(wheel_model)
 
     def test_part_html_table(self):
         part = self.project.part('Front Wheel')
@@ -204,21 +219,21 @@ class TestParts(TestBetamax):
 
     # version 1.1.3 and later
     def test_retrieve_parent_of_part(self):
-        frame = self.project.part('Frame')  # type:Part
+        frame = self.project.part('Frame')  # type: Part2
         self.assertTrue(hasattr(frame, 'parent_id'))
         parent_of_frame = frame.parent()
         self.assertIsInstance(parent_of_frame, type(frame))
 
     def test_retrieve_children_of_part(self):
-        bike = self.project.part('Bike')  # type:Part
-        self.assertIsInstance(bike, (Part, Part2))
+        bike = self.project.part('Bike')  # type: Part2
+        self.assertIsInstance(bike, Part2)
         children_of_bike = bike.children()
         self.assertIsInstance(children_of_bike, (PartSet, list))
         self.assertTrue(len(children_of_bike) >= 1)  # eg. Wheels ...
 
     def test_retrieve_siblings_of_part(self):
         """Test if the siblings of a part is the same as the children of the parent of the part"""
-        frame = self.project.part('Frame')  # type:Part
+        frame = self.project.part('Frame')  # type: Part2
         siblings_of_frame = frame.siblings()
         self.assertIsInstance(siblings_of_frame, PartSet)
         self.assertTrue(len(siblings_of_frame) >= 1)  # eg. Wheels ...
@@ -242,7 +257,7 @@ class TestParts(TestBetamax):
     # new in 1.5+
     def test_edit_part_instance_name(self):
         front_fork = self.project.part('Front Fork')
-        front_fork_oldname = str(front_fork.name)
+        front_fork_old_name = str(front_fork.name)
         front_fork.edit(name='Front Fork - updated')
 
         front_fork_updated = self.project.part('Front Fork - updated')
@@ -254,11 +269,11 @@ class TestParts(TestBetamax):
             front_fork.edit(name=True)
 
         # teardown
-        front_fork.edit(name=front_fork_oldname)
+        front_fork.edit(name=front_fork_old_name)
 
     def test_edit_part_instance_description(self):
         front_fork = self.project.part('Front Fork')
-        front_fork_olddescription = str(front_fork._json_data.get('description'))
+        front_fork_old_description = str(front_fork._json_data.get('description'))
         front_fork.edit(description='A normal Front Fork')
 
         front_fork_updated = self.project.part('Front Fork')
@@ -268,7 +283,7 @@ class TestParts(TestBetamax):
             front_fork.edit(description=42)
 
         # teardown
-        front_fork.edit(description=front_fork_olddescription)
+        front_fork.edit(description=front_fork_old_description)
 
     def test_edit_part_model_name(self):
         front_fork = self.project.model('Front Fork')
@@ -285,13 +300,12 @@ class TestParts(TestBetamax):
     def test_create_model(self):
         bike = self.project.model('Bike')
         pedal = self.project.create_model(bike, 'Pedal', multiplicity=Multiplicity.ONE)
+        self._part = pedal
         pedal_model = self.project.model('Pedal')
+
         self.assertTrue(pedal.id, pedal_model.id)
         self.assertTrue(pedal._json_data['multiplicity'], 'ONE')
 
-        # teardown
-        pedal_model.delete()
-
     def test_add_proxy_to(self):
         catalog_container = self.project.model(name__startswith='Catalog')
         bearing_catalog_model = catalog_container.add_model('Bearing', multiplicity=Multiplicity.ZERO_MANY)
@@ -391,7 +405,7 @@ class TestParts(TestBetamax):
 
     # new in 1.12
     def test_retrieve_children_of_part_with_additional_arguments(self):
-        bike = self.project.part('Bike')  # type:Part
+        bike = self.project.part('Bike')  # type: Part2
         self.assertIsInstance(bike, Part2)
         children_of_bike = bike.children(name__icontains='Wheel')
         self.assertIsInstance(children_of_bike, PartSet)
@@ -399,7 +413,7 @@ class TestParts(TestBetamax):
 
     def test_retrieve_siblings_of_part_with_additional_arguments(self):
         """Test if the siblings of a part is the same as the children of the parent of the part"""
-        frame = self.project.part('Frame')  # type:Part
+        frame = self.project.part('Frame')  # type: Part2
         siblings_of_frame = frame.siblings(name__icontains='Wheel')
         self.assertIsInstance(siblings_of_frame, PartSet)
         self.assertTrue(len(siblings_of_frame) >= 1)  # eg. Wheels ...
@@ -409,34 +423,31 @@ class TestParts(TestBetamax):
         # setUp
         model_name = 'Seat'
         seat = self.project.model(model_name)
-        seat.clone()
+        self._part = seat.clone()
 
         # testing
         clone_seat_model = self.project.model('CLONE - {}'.format(model_name))
         self.assertTrue(clone_seat_model)
 
-        # tearDown
-        clone_seat_model.delete()
-
     def test_clone_instance(self):
         instance_name = 'Front Wheel'
         wheel = self.project.part(instance_name)
-        wheel.clone()
+        self._part = wheel.clone()
 
         # testing
         clone_spoke_instance = self.project.part('CLONE - {}'.format(instance_name))
         self.assertTrue(clone_spoke_instance)
 
-        # tearDown
-        clone_spoke_instance.delete()
-
     def test_clone_instance_with_multiplicity_violation(self):
         instance_name = 'Seat'
         seat = self.project.part(instance_name)
 
+        seat_model = seat.model()
+
         # testing
+        self.assertEqual(Multiplicity.ONE, seat_model.multiplicity)
         with self.assertRaises(APIError):
-            seat.clone()
+            self._part = seat.clone()
 
 
 @skipIf(not TEST_FLAG_IS_PIM3, reason="This tests is designed for PIM version 3, expected to fail on older PIM3")
